
## This script shoudl be run TWICE - once with 'ploidy <- 4', then
## again with 'ploidy <- 6' (see below).  Each run may take a few
## hours.  Then, run 'Sim2.R' to plot the simulation results.

## R packages available from CRAN - see help for R function
## install.packages(), or your R IDE's package installation system.
##
require(PolyPatEx)
require(dplyr)
require(ggplot2)
##  Assumed to be in the current R working directory
source("SimulationFunctions.R")

## Ploidy
ploidy <- 4
##
## Number of alleles to simulate per locus
nAlleles <- 5
##
## Number of mothers
nMothers <- 5
##
## Number of seedlings per mother
nProgenyPerMother <- 10
##
## Number of potential fathers
nCandidates <- 30
##
nSim <- 100

set.seed(5280)
##
## Numbers of Loci to simulate
numLoci <- c(5,7,10,15,20,25)
##
gAvgPotentialsFound <- rep(NA,length.out=length(numLoci))
pAvgPotentialsFound <- rep(NA,length.out=length(numLoci))
gSDPotentialsFound <- rep(NA,length.out=length(numLoci))
pSDPotentialsFound <- rep(NA,length.out=length(numLoci))
g975PotentialsFound <- rep(NA,length.out=length(numLoci))
p975PotentialsFound <- rep(NA,length.out=length(numLoci))
g025PotentialsFound <- rep(NA,length.out=length(numLoci))
p025PotentialsFound <- rep(NA,length.out=length(numLoci))
g50PotentialsFound <- rep(NA,length.out=length(numLoci))
p50PotentialsFound <- rep(NA,length.out=length(numLoci))
g75PotentialsFound <- rep(NA,length.out=length(numLoci))
p75PotentialsFound <- rep(NA,length.out=length(numLoci))
g25PotentialsFound <- rep(NA,length.out=length(numLoci))
p25PotentialsFound <- rep(NA,length.out=length(numLoci))
##
for (j in 1:length(numLoci))
{
  cat("\nSimulating nLoci=",numLoci[j],"\n")
  gNumPotentialsFoundSim <- NULL
  pNumPotentialsFoundSim <- NULL
  for (i in 1:nSim)
    {
      cat("\nIteration",i,"\n")
      ss <- simulateAlleleDataset(ploidy = ploidy,
                                  nLoci = numLoci[j],
                                  nAlleles = nAlleles,
                                  nCandidates = nCandidates,
                                  nMothers = nMothers,
                                  nProgenyPerMother = nProgenyPerMother,
                                  simName = "Simulation")
      aData <- ss$aData
      progenyFathers <- ss$progenyFathers
      ## Preprocess allele dataset, generate allelic phenotype dataset from
      ## genotypic dataset.  Perform exclusion, extract father ID info.
      gData <- preprocessData(aData,
                              numLoci=numLoci[j],
                              ploidy=ploidy,
                              dataType="genotype",
                              dioecious=TRUE,
                              mothersOnly=TRUE)
      pData <- convertToPhenot(gData)
      ##
      gEx <- genotPPE(gData)
      pEx <- phenotPPE(pData)
      ##
      gFthr <- potentialFatherIDs(gEx,VLTMin=numLoci[j])
      pFthr <- potentialFatherIDs(pEx,VLTMin=numLoci[j])
      ##
      ## Was the true father detected?
      gFatherFound <- rep(NA,length=length(progenyFathers))
      names(gFatherFound) <- names(progenyFathers)
      for (pp in names(progenyFathers))
        {
          gFatherFound[pp] <- with(gFthr,
                                   progenyFathers[[pp]] %in%
                                   potentialFather[Progeny==pp])
        }
      ##    gFatherFound
      stopifnot(all(gFatherFound))
      ##
      ## How many fathers detected?
      gNumPotentialsFound <- gFthr %>%
        group_by(Progeny) %>%
          summarise(numPotentials=sum(!is.na(potentialFather)))
      ##
      gNumPotentialsFoundSim <- c(gNumPotentialsFoundSim,
                                  gNumPotentialsFound$numPotentials)
      ##
      pFatherFound <- rep(NA,length=length(progenyFathers))
      names(pFatherFound) <- names(progenyFathers)
      for (pp in names(progenyFathers))
        {
          pFatherFound[pp] <- with(pFthr,
                                   progenyFathers[[pp]] %in%
                                   potentialFather[Progeny==pp])
        }
      stopifnot(all(pFatherFound))
      ##
      pNumPotentialsFound <- pFthr %>%
        group_by(Progeny) %>%
          summarise(numPotentials=sum(!is.na(potentialFather)))
      pNumPotentialsFoundSim <- c(pNumPotentialsFoundSim,
                                  pNumPotentialsFound$numPotentials)
    }
  stopifnot(!any(is.na(gNumPotentialsFoundSim)))
  stopifnot(!any(is.na(pNumPotentialsFoundSim)))
  gAvgPotentialsFound[j]  <- mean(gNumPotentialsFoundSim,na.rm=TRUE)
  pAvgPotentialsFound[j]  <- mean(pNumPotentialsFoundSim,na.rm=TRUE)
  gSDPotentialsFound[j]  <- sd(gNumPotentialsFoundSim,na.rm=TRUE)
  pSDPotentialsFound[j]  <- sd(pNumPotentialsFoundSim,na.rm=TRUE)
  g975PotentialsFound[j]  <- quantile(gNumPotentialsFoundSim,
                                            probs=0.975,na.rm=TRUE)
  p975PotentialsFound[j]  <- quantile(pNumPotentialsFoundSim,
                                            probs=0.975,na.rm=TRUE)
  g025PotentialsFound[j]  <- quantile(gNumPotentialsFoundSim,
                                            probs=0.025,na.rm=TRUE)
  p025PotentialsFound[j]  <- quantile(pNumPotentialsFoundSim,
                                            probs=0.025,na.rm=TRUE)
  g50PotentialsFound[j]  <- quantile(gNumPotentialsFoundSim,
                                            probs=0.50,na.rm=TRUE)
  p50PotentialsFound[j]  <- quantile(pNumPotentialsFoundSim,
                                            probs=0.50,na.rm=TRUE)
  g75PotentialsFound[j]  <- quantile(gNumPotentialsFoundSim,
                                            probs=0.75,na.rm=TRUE)
  p75PotentialsFound[j]  <- quantile(pNumPotentialsFoundSim,
                                            probs=0.75,na.rm=TRUE)
  g25PotentialsFound[j]  <- quantile(gNumPotentialsFoundSim,
                                            probs=0.25,na.rm=TRUE)
  p25PotentialsFound[j]  <- quantile(pNumPotentialsFoundSim,
                                            probs=0.25,na.rm=TRUE)
}

## Put simulation results together into a dataframe for plotting:
gdd <- data.frame(numLoci = numLoci,
                  avgPotentialsFound = gAvgPotentialsFound,
                  SDPotentialsFound = gSDPotentialsFound,
                  Q975PotentialsFound = g975PotentialsFound,
                  Q025PotentialsFound = g025PotentialsFound,
                  Q50PotentialsFound = g50PotentialsFound,
                  Q75PotentialsFound = g75PotentialsFound,
                  Q25PotentialsFound = g25PotentialsFound)
gdd$dataType <- "Genotype"
attr(gdd,which="meta") <- "no missing data or genotyping errors"
pdd <- data.frame(numLoci = numLoci,
                  avgPotentialsFound = pAvgPotentialsFound,
                  SDPotentialsFound = pSDPotentialsFound,
                  Q975PotentialsFound = p975PotentialsFound,
                  Q025PotentialsFound = p025PotentialsFound,
                  Q50PotentialsFound = p50PotentialsFound,
                  Q75PotentialsFound = p75PotentialsFound,
                  Q25PotentialsFound = p25PotentialsFound)
pdd$dataType <- "Allelic phenotype"
attr(pdd,which="meta") <- "No missing data or genotyping errors"
dd <- rbind(gdd,pdd)
attr(dd,which="ploidy") <- ploidy
attr(dd,which="nAlleles") <- nAlleles
attr(dd,which="nMothers") <- nMothers
attr(dd,which="nProgenyPerMother") <- nProgenyPerMother
attr(dd,which="nCandidates") <- nCandidates
attr(dd,which="meta") <- "No genotyping errors or missings in simulated data"
##
save(dd,file=paste0("simulation_ploidy_",ploidy,".RData"))

